//
//  Renderer.swift
//  Pods-Runner
//
//  Created by Zhenya Knizhnik on 02.09.19.
//

import Foundation
import ScanbotSDK
import Flutter

final class Renderer {
    func renderPDF(from pages: [SBSDKUIPage],
                   pageSize: SBSDKPDFRendererPageSize = .custom,
                   pageOrientation: SBSDKPDFRendererPageOrientation = .auto,
                   pdfAttributes: [String: Any]? = nil,
                   result: @escaping FlutterResult) {
        
        if pages.isEmpty {
            result(RendererError.noPagePassedPdfError.value)
            return
        }
        DispatchQueue.global(qos: .background).async {
            let storage = SBSDKIndexedImageStorage.temporaryStorage(for: pages)
            let outputURL = PluginPreferences.shared.newStorageCustomFileURL(extensionString: "pdf")
            
            // Try to change to SBSDKPDFUIRenderer after update
            let options = SBSDKPDFRendererOptions(pageSize: pageSize, 
                                                  pageOrientation: pageOrientation, 
                                                  ocrConfiguration: nil)
            SBSDKPDFRenderer(options: options).renderImageStorage(storage,
                                                                  indexSet: nil,
                                                                  output: outputURL) { finished, error in
                if let pdfAttributes {
                    SBSDKPDFMetadataProcessor.writeMetadata(pdfAttributes.metadata,
                                                            withPDFFrom: outputURL,
                                                            to: outputURL,
                                                            removeCurrentMeta: false)
                }
                self.pdfProcessCompletion(finished: finished, error: error, pdfURL: outputURL, result: result)
                storage.removeAllImages()
            }
        }
    }
    
    func renderPDF(from pages: [SBSDKUIPage],
                   pageSize: SBSDKPDFRendererPageSize = .custom, 
                   pageOrientation: SBSDKPDFRendererPageOrientation,
                   pdfAttributes: [String: Any]? = nil,
                   pdfOutputURL: URL?,
                   cleanupStorageAfterPerform: Bool = true,
                   result: @escaping (URL?) -> ()) {
        
        guard let pdfOutputURL, !pages.isEmpty else {
            result(nil)
            return
        }
        
        DispatchQueue.global(qos: .background).async {
            let storage = SBSDKIndexedImageStorage.temporaryStorage(for: pages)
            let options = SBSDKPDFRendererOptions(pageSize: pageSize,
                                                  pageOrientation: pageOrientation,
                                                  ocrConfiguration: nil)
            SBSDKPDFRenderer(options: options).renderImageStorage(storage, 
                                                                  indexSet: nil, 
                                                                  output: pdfOutputURL) { finished, error in
                if error != nil {
                    result(nil)
                } else {
                    if let pdfAttributes {
                        SBSDKPDFMetadataProcessor.writeMetadata(pdfAttributes.metadata,
                                                                withPDFFrom: pdfOutputURL,
                                                                to: pdfOutputURL,
                                                                removeCurrentMeta: false)
                    }
                    result(pdfOutputURL)
                }
                if cleanupStorageAfterPerform {
                    storage.removeAllImages()
                }
            }
        }
    }
    
    private func pdfProcessCompletion(finished: Bool,
                                      error: Error?,
                                      pdfURL: URL?,
                                      result: @escaping FlutterResult) {
        DispatchQueue.main.async {
            if let error = error {
                result(RendererError.generic(message: error.localizedDescription).value)
            } else if finished, let outputURL = pdfURL {
                result(outputURL.absoluteString)
            } else {
                result(RendererError.unknownPdfError.value)
            }
        }
    }
    
    func renderTIFF(from pages: [SBSDKUIPage],
                    options: TIFFOptions,
                    result: @escaping FlutterResult) {
        
        if pages.isEmpty {
            result(RendererError.noPagePassedTiffError.value)
            return
        }
        DispatchQueue.global(qos: .background).async {
            let outputURL = PluginPreferences.shared.newStorageCustomFileURL(extensionString: "tiff")
            let imageURLs = pages.compactMap({ $0.documentImageURL() })
            
            let params = SBSDKTIFFImageWriterParameters.default()
            
            params.dpi = options.dpi
            params.binarize = options.isBinarized
            params.compression = options.compression
            
            var success: Bool = false
            // If we have an encrypter, we need to decrypt images before processing.
            if let pdfEncrypter = ScanbotUI.defaultPDFEncrypter() {
                // We are using PDF encrypter, as it should be the same decrypter for everything.
                // We are mimicking android SDK behavior this way.
                let decryptedImages = self.decryptedImages(from: imageURLs, encrypter: pdfEncrypter)
                success = SBSDKTIFFImageWriter.writeTIFF(decryptedImages,
                                                         fileURL: outputURL,
                                                         encrypter: pdfEncrypter,
                                                         parameters: params)
            } else {
                success = SBSDKTIFFImageWriter.writeTIFF(from: imageURLs,
                                                         fileURL: outputURL,
                                                         parameters: params)
            }
            
            DispatchQueue.main.async {
                if success {
                    result(outputURL.absoluteString)
                } else {
                    result(RendererError.unknownTiffError.value)
                }
            }
        }
    }
    
    private func decryptedImages(from urls: [URL], encrypter: SBSDKStorageCrypting) -> [UIImage] {
        var resultedImages: [UIImage] = []
        for url in urls {
            if let data = try? Data(contentsOf: url), let image = UIImage(data: encrypter.decryptData(data)) {
                resultedImages.append(image)
            }
        }
        return resultedImages
    }
}
