//
//  CroppingNativeView.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 04.01.23.
//

import Foundation
import Flutter
import ScanbotSDK

class CroppingNativeView: NSObject, FlutterPlatformView, CallErrorHandler, SBSDKImageEditingViewControllerDelegate {
    
    private var viewController: SBSDKImageEditingViewController?
    private var containerView: UIView
    private let configuration: [String: Any?]?
    private var croppingMethodChannel: FlutterMethodChannel?
    private var processor: SBSDKDocumentDetector?
    private var currentPage: SBSDKUIPage?
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger messenger: FlutterBinaryMessenger
    ) {
        croppingMethodChannel = FlutterMethodChannel(name: "scanbot_sdk_cropping_\(viewId)", binaryMessenger: messenger)
        containerView = UIView()
        configuration = args as? [String: Any?]
        processor = SBSDKDocumentDetector(mode: SBSDKDocumentDetector.defaultDetectorMode())
        
        super.init()
        self.createNativeView(view: self.containerView)
        self.croppingMethodChannel?.setMethodCallHandler({ (call, result) in
            self.handleCameraCall(call: call, result: result)
        })
    }
    
    func view() -> UIView {
        return containerView
    }
    
    func createNativeView(view: UIView) {
        if !Scanbot.isLicenseValid() {
            DispatchQueue.main.async { [weak self] in
                guard let callHandler = self?.croppingMethodChannel else {
                    fatalError("callHandler cannot be nil")
                }
                LicenseHandler.sendLicenseErrorToDetector(callHandler)
            }
            return
        }
        
        let rootViewController = UIViewController()
        rootViewController.view = view
        rootViewController.view.backgroundColor = UIColor.clear
        viewController = SBSDKImageEditingViewController(parentViewController: rootViewController, containerView: view)
        viewController?.delegate = self
        
        if let arguments = configuration {
            setInitialConfiguration(arguments: arguments)
        }
        
    }
    
    func setInitialConfiguration(arguments: [String: Any?]) {
        guard let viewController = viewController else {
            return
        }
        let configuration = CroppingNativeViewConfiguration(arguments)
        
        guard let page = configuration.page else {return }
        currentPage = page
        
        guard let image = page.originalPreviewImage() else {return }
        viewController.view.backgroundColor = UIColor.clear
        
        if let edgeLineWidth = configuration.edgeLineWidth {
            viewController.edgeLineWidth = edgeLineWidth
        }
        if let edgeColor = configuration.edgeColor {
            viewController.edgeColor = edgeColor
        }
        if let anchorPointsColor = configuration.anchorPointsColor {
            viewController.anchorPointsColor = anchorPointsColor
        }
        if let edgeColorOnLine = configuration.edgeColorOnLine {
            viewController.magneticEdgeColor = edgeColorOnLine
        }
        if let borderInsets = configuration.borderInsets {
            viewController.contentInsets = borderInsets
        }
        
        viewController.processingQueue = DispatchQueue.main
        viewController.polygon = page.polygon ?? SBSDKPolygon()
        viewController.image = image
    }
    
    func handleCameraCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let viewController = viewController,
              let page = currentPage else {
            returnGuardError(result: result)
            return
        }
        
        switch call.method {
        case "rotateCW":
            viewController.rotateInputImageClockwise(true, animated: true)
            let size = viewController.image.size
            page.polygon?.rotateCW(UInt(1), with: size)
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.1, execute: { [weak self] in
                self?.returnSuccess(result: result)
            })
            return
        case "setNewPage":
            let dictionary = call.arguments as! [String: Any]
            guard let page = SBSDKUIPage.init(from: dictionary) else {
                returnGuardError(result: result)
                return
            }
        
            guard let image = page.originalPreviewImage() else {
                returnGuardError(result: result)
                return
            }
            
            viewController.polygon = page.polygon ?? SBSDKPolygon()
            viewController.image = image
            viewController.rotations = 0
        case "rotateCCW":
            viewController.rotateInputImageClockwise(false, animated: true)
            let size = viewController.image.size
            page.polygon?.rotateCCW(UInt(1), with: size)
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.1, execute: { [weak self] in
                self?.returnSuccess(result: result)
            })
            return
        case "getEditedPolygon":
            guard let polygon = viewController.polygon else {
                returnGuardError(result: result)
                return
            }
            let rotation = viewController.rotations
            result(getJsonForPolygonData(polygon: polygon, rotation: rotation).encodedString)
            return
        case "reset":
            let resetPolygon = SBSDKPolygon.init()
            viewController.polygon = resetPolygon
        case "detect":
            let image = viewController.image
            let result = processor?.detectDocumentPolygon(on: image,
                                                          visibleImageRect: CGRect.zero,
                                                          smoothingEnabled: false,
                                                          useLiveDetectionParameters: false)
            viewController.polygon = result?.polygon
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        returnSuccess(result: result)
    }
    
    func getJsonForPolygonData(polygon: SBSDKPolygon, rotation: Int) -> [String: Any] {
        var croppingResult: [String: Any] = [:]
        croppingResult["polygon"] = polygon.encodedDictionary
        croppingResult["rotationTimesCw"] = rotation
        return croppingResult
    }
    
    func returnSuccess(result: @escaping FlutterResult) {
        result("success")
    }
    
    func returnGuardError(result: @escaping FlutterResult) {
        result(CroppingNativeViewError.nullParameter.value)
    }
    
    func imageEditingViewControllerDidChangePolygon(_ editingViewController: SBSDKImageEditingViewController) {
        if editingViewController.polygon?.isSimilar(to: SBSDKPolygon()) == false { }
    }
}
