//
//  MedicalCertificate.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 14.02.23.
//

import Foundation
import ScanbotSDK

class MedicalCertificateScanner: NSObject,
                                 ScannerDelegate,
                                 SBSDKMedicalCertificateScannerViewControllerDelegate,
                                 CallErrorHandler {
    
    private var viewController: SBSDKMedicalCertificateScannerViewController?
    
    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?
    private var configuration: [String: Any?]
    private var cachedResult: SBSDKMedicalCertificateRecognizerResult?
    internal var shouldRunDetection: Bool = true
    private var shouldPauseDetectionOnResultReturn: Bool = false
    
    /// Recognition of the Patient Information block. [true] if enabled.
    private var recognizePatientInfo: Bool = true
    
    /// Reading data from barcode, if it's present on document, this could make data recognition more accurate. [true] if enabled.
    private var recognizeBarcode: Bool = true
    
    /// Make high-res snapshot of the image after first successful medical certificate detection and recognise medical certificate fields on it
    private var captureHighResolutionImage: Bool = false
    
    init(
        cameraMethodChannel: FlutterMethodChannel?,
        detectorChannel: FlutterMethodChannel?,
        finderChannel: FlutterMethodChannel?,
        configuration: [String: Any?]
    ) {
        self.cameraMethodChannel = cameraMethodChannel
        self.detectorChannel = detectorChannel
        self.finderChannel = finderChannel
        self.configuration = configuration
    }
    
    func createNativeView(view: UIView) {
        if !Scanbot.isLicenseValid() {
            DispatchQueue.main.async { [weak self] in
                guard let callHandler = self?.detectorChannel else {
                    fatalError("callHandler cannot be nil")
                }
                LicenseHandler.sendLicenseErrorToDetector(callHandler)
            }
            return
        }
        
        viewController = SBSDKMedicalCertificateScannerViewController()
        viewController?.delegate = self
        viewController?.attachFlutterView(view)
        _ = viewController?.view
        
        setMcScannerConfiguration(configuration: configuration)
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnCameraOpenMethod, arguments: "")
    }
    
    func attach(view: UIView) {
        createNativeView(view: view)
        self.detectorChannel?.setMethodCallHandler({ (call, result) in
            self.handleDetectorCall(call: call, result: result)
        })
    }
    
    func setFlash(enabled: Bool) {
        viewController?.isFlashLightEnabled = enabled
    }
    
    func isFlashAvailable() -> Bool {
        return viewController?.isFlashLightAvailable ?? false
    }

    func medicalCertificateScannerViewControllerShouldRecognize(_ controller: SBSDKMedicalCertificateScannerViewController) -> Bool {
        return shouldRunDetection && !shouldPauseDetectionOnResultReturn
    }
  
    func medicalCertificateScannerViewController(_ controller: SBSDKMedicalCertificateScannerViewController, didChange state: SBSDKMedicalCertificateScannerState) {
        switch(state){
        case .capturing: cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnShowHeavyProcessing, arguments: true)
        default: break
        }
    }
    
    func medicalCertificateScannerViewController(_ controller: SBSDKMedicalCertificateScannerViewController,
                                                 didRecognizeMedicalCertificate result: SBSDKMedicalCertificateRecognizerResult) {
        
        shouldPauseDetectionOnResultReturn = true
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnShowHeavyProcessing, arguments: true)
        cachedResult = result
        var imageUri: URL?
        if self.captureHighResolutionImage {
            if let image = result.image {
                guard let uuid = SBSDKUIPageFileStorage.default().add(image) else { return }
                imageUri = SBSDKUIPageFileStorage.default().imageURL(withPageFileID: uuid, pageFileType: .original)
            }
            
        }
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnShowHeavyProcessing, arguments: false)
        sendDetectedResult(result, imageUri?.standardizedFileURL.absoluteString) { [weak self] in
            self?.shouldPauseDetectionOnResultReturn = false
        }
    }
    
    func didChangeViewFinderRect(rect: CGRect) {
        finderChannel?.invokeMethod("finderSizeChanged", arguments: rect.rectDictionary)
    }
    
    private func sendDetectedResult(_ result: SBSDKMedicalCertificateRecognizerResult,
                                    _ imageImageURI: String?,
                                    completion: @escaping () -> Void) {
        var resultDict =  result.toSdkJson()
        resultDict["croppedDocumentURI"] = imageImageURI
        
        DispatchQueue.main.async { [weak self] in
            guard let callHandler = self?.detectorChannel else {
                fatalError("callHandler cannot be nil")
            }
            callHandler.invokeMethod(MethodChannelConstants.kOnMedicalCertificateDetected,
                                     arguments: resultDict.encodedString){ _ in
                completion()
            }
        }
    }
    
    func setMcScannerConfiguration(configuration: [String: Any?]?) {
        
        viewController?.viewFinderRectDidChangeHandler = { finderRect in
            self.didChangeViewFinderRect(rect: finderRect)
        }
        
        let medicalCertificateConfiguration = MedicalCertificateConfiguration(configuration)
        
        if let flashEnabled = medicalCertificateConfiguration.flashEnabled {
            viewController?.isFlashLightEnabled = flashEnabled
        }
        
        if let zoomConfiguration = medicalCertificateConfiguration.zoomConfiguration {
            viewController?.zoomConfiguration = zoomConfiguration
        }
        
        if let finderConfiguration = medicalCertificateConfiguration.viewFinderConfiguration {
            viewController?.viewFinderConfiguration = finderConfiguration
        }
        
        if let scannerConfiguration = medicalCertificateConfiguration.scannerConfiguration {
            
            if let captureHighResolutionImage = scannerConfiguration.captureHighResolutionImage {
                viewController?.captureHighResolutionImage = captureHighResolutionImage
                self.captureHighResolutionImage = captureHighResolutionImage
            }
            
            if let extractPatientInfo = scannerConfiguration.extractPatientInfo {
                viewController?.extractPatientInfo = extractPatientInfo
                self.recognizePatientInfo = extractPatientInfo
            }
            
            if let recognizeBarcode = scannerConfiguration.recognizeBarcode {
                viewController?.extractBarcode = recognizeBarcode
                self.recognizeBarcode = recognizeBarcode
            }
        }
    }
    
    func stopPreview() {
        viewController?.freezeCamera()
    }
    
    func resumePreview() {
        viewController?.unfreezeCamera()
    }
    
    func handleDetectorCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "enable": shouldRunDetection = true
        case "disable": shouldRunDetection = false
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        returnSuccess(result: result) // always return some result to the flutter
    }
    
    func returnSuccess(result: @escaping FlutterResult) {
        result("success")
    }
}
