//
//  ErrorHandler.swift
//  scanbot_sdk
//
//  Created by Seifeddine Bouzid on 22.03.23.
//

import Foundation
import ScanbotSDK

protocol FlutterErrorHandler {
    func flutterError(code: Int, message: String, details: ErrorTypes?) -> FlutterError
}

extension FlutterErrorHandler {
    func flutterError(code: Int, message: String, details: ErrorTypes?) -> FlutterError {
        return FlutterError(code: "\(code)",
                            message: "ERROR_IOS: \(message)",
                            details: details != nil ? details!.value : nil)
    }
}

enum ErrorTypes {
    
    case pdf
    case tiff
    case ocr 
    case any(_: Any?)
    
    var value: Any? {
        switch self {
        case .pdf: 
            return "Rendering PDF error"
        case .tiff: 
            return "Rendering TIFF error"
        case .ocr: 
            return "OCR error"
        case .any(let detail): 
            return detail
        }
    }
}

enum CallError: FlutterErrorHandler {
    
    case argumentsError(method: String, details: Any?)
    case unknownCallError(method: String)
    case customError(message: String, details: Any?)
    
    var value: FlutterError {
        switch self {
        case .argumentsError(let method, let details):
            return self.flutterError(code: 1001,
                                     message: "\(method) was called with wrong arguments",
                                     details: ErrorTypes.any(details))
        case .unknownCallError(let method):
            return self.flutterError(code: 1002,
                                     message: "\(method) is unknown or can't be handled",
                                     details: nil)
        case .customError(let message, let details):
            return self.flutterError(code: 1003, message: message, details: ErrorTypes.any(details))
        }
    }
}

enum RendererError: FlutterErrorHandler {
    
    case generic(message: String)
    case unknownPdfError
    case unknownTiffError
    case noPagePassedPdfError
    case noPagePassedTiffError
    
    var value: FlutterError {
        switch self {
        case .generic(let message):
            return self.flutterError(code: 1006, message: message, details: .pdf)
        case .unknownPdfError:
            return self.flutterError(code: 1007, message: "Unknown rendering error", details: .pdf)
        case .unknownTiffError:
            return self.flutterError(code: 1008, message: "Unknown rendering error", details: .tiff)
        case .noPagePassedPdfError:
            return self.flutterError(code: 1010, message: "No pages passed", details: .pdf)
        case .noPagePassedTiffError:
            return self.flutterError(code: 1011, message: "No pages passed", details: .tiff)
        }
    }
}

enum OCRPerformerError: FlutterErrorHandler {
    
    case ocrConfig
    case generic(message: String)
    case unknown(for: ErrorTypes)
    
    var value: FlutterError {
        switch self {
        case .ocrConfig:
            return self.flutterError(code: 1011, message: "OCRConfig getting error", details: nil)
        case .generic(let message):
            return self.flutterError(code: 1012, message: message, details: .ocr)
        case .unknown(let details):
            return self.flutterError(code: 1013, message: "Unknown rendering error", details: details)
        }
    }
}

enum CroppingNativeViewError: FlutterErrorHandler {
    
    case nullParameter
    
    var value: FlutterError {
        switch self {
        case .nullParameter:
            let message = "Can't apply method due to the reason: One of the parameters is null"
            return self.flutterError(code: 1005, message: message, details: nil)
        }
    }
}
