import 'dart:io';
import 'dart:math' as math;

import 'package:flutter/material.dart';

// ignore: must_be_immutable
class ShutterButton extends StatelessWidget {
  late Widget button;
  final VoidCallback? onPressed;

  ShutterButton(
      {Key? key,
      required this.onPressed,
      Color primaryColor = Colors.white,
      Color accentColor = Colors.teal,
      double innerArcSpeedMultiplier = 1.0,
      double outerArcSpeedMultiplier = 1.0,
      double? animatedLineStrokeWidth,
      double outerCircleStrokeWidth = 3,
      double? outerAnimatedArcRadius,
      double buttonSize = 56,
      double iconSize = 24,
      bool autosnappingMode = true})
      : super(key: key) {
    button = Platform.isAndroid
        ? AndroidShutterButton(
            buttonSize: buttonSize,
            autosnappingMode: autosnappingMode,
            innerArcSpeedMultiplier: innerArcSpeedMultiplier,
            outerArcSpeedMultiplier: outerArcSpeedMultiplier,
            outerAnimatedArcRadius: outerAnimatedArcRadius ?? 12,
            animatedLineStrokeWidth: animatedLineStrokeWidth ?? 2,
            accentColor: accentColor,
            iconSize: iconSize,
            primaryColor: primaryColor,
            onPressed: onPressed,
          )
        : IosShutterButton(
            buttonSize: buttonSize,
            outerCircleStrokeWidth: outerCircleStrokeWidth,
            accentColor: accentColor,
            innerArcSpeedMultiplier: innerArcSpeedMultiplier,
            outerArcSpeedMultiplier: outerArcSpeedMultiplier,
            outerAnimatedArcRadius: outerAnimatedArcRadius ??
                buttonSize / 2 - (animatedLineStrokeWidth ?? 2) / 2,
            animatedLineStrokeWidth: animatedLineStrokeWidth ?? 2,
            autosnappingMode: autosnappingMode,
            onPressed: onPressed,
          );
  }

  @override
  Widget build(BuildContext context) {
    return button;
  }
}

class IosShutterButton extends StatelessWidget {
  const IosShutterButton({
    Key? key,
    required this.buttonSize,
    required this.outerCircleStrokeWidth,
    required this.accentColor,
    required this.innerArcSpeedMultiplier,
    required this.outerArcSpeedMultiplier,
    required this.outerAnimatedArcRadius,
    required this.animatedLineStrokeWidth,
    required this.autosnappingMode,
    required this.onPressed,
  }) : super(key: key);

  final VoidCallback? onPressed;
  final double buttonSize;
  final double outerCircleStrokeWidth;
  final Color accentColor;
  final double innerArcSpeedMultiplier;
  final double outerArcSpeedMultiplier;
  final double outerAnimatedArcRadius;
  final double animatedLineStrokeWidth;
  final bool autosnappingMode;

  @override
  Widget build(BuildContext context) {
    return SizedBox(
        width: buttonSize + outerCircleStrokeWidth * 4,
        height: buttonSize + outerCircleStrokeWidth * 4,
        child: Stack(
          children: [
            CustomPaint(
                painter: ArcPainter(
                    buttonSize / 2 + outerCircleStrokeWidth * 2,
                    outerCircleStrokeWidth,
                    0,
                    360,
                    accentColor,
                    ValueNotifier(0)),
                size: Size(buttonSize + outerCircleStrokeWidth * 4,
                    buttonSize + outerCircleStrokeWidth * 4)),
            Center(
              child: SizedBox(
                width: buttonSize,
                height: buttonSize,
                child: autosnappingMode
                    ? InkWell(
                        onTap: onPressed,
                        child: ShutterAnimation(
                            gapBetweenArcs: outerCircleStrokeWidth * 2,
                            innerArcSpeedMultiplier: innerArcSpeedMultiplier,
                            outerArcSpeedMultiplier: outerArcSpeedMultiplier,
                            outerArcRadius: outerAnimatedArcRadius,
                            animatedLineStrokeWidth: animatedLineStrokeWidth,
                            accentColor: accentColor),
                      )
                    : ElevatedButton(
                        onPressed: onPressed,
                        child: Container(),
                        style: ElevatedButton.styleFrom(
                          foregroundColor: accentColor,
                          backgroundColor: accentColor,
                          elevation: 0,
                          enableFeedback: false,
                          shape: CircleBorder(
                              side: BorderSide(color: accentColor, width: 3)),
                          padding: const EdgeInsets.all(0),
                        ),
                      ),
              ),
            ),
          ],
        ));
  }
}

class AndroidShutterButton extends StatelessWidget {
  const AndroidShutterButton({
    Key? key,
    required this.buttonSize,
    required this.autosnappingMode,
    required this.innerArcSpeedMultiplier,
    required this.outerArcSpeedMultiplier,
    required this.outerAnimatedArcRadius,
    required this.animatedLineStrokeWidth,
    required this.accentColor,
    required this.iconSize,
    required this.primaryColor,
    required this.onPressed,
  }) : super(key: key);

  final VoidCallback? onPressed;
  final double buttonSize;
  final bool autosnappingMode;
  final double innerArcSpeedMultiplier;
  final double outerArcSpeedMultiplier;
  final double outerAnimatedArcRadius;
  final double animatedLineStrokeWidth;
  final Color accentColor;
  final double iconSize;
  final Color primaryColor;

  @override
  Widget build(BuildContext context) {
    return SizedBox(
        width: buttonSize,
        height: buttonSize,
        child: ElevatedButton(
          child: Center(
            child: autosnappingMode
                ? ShutterAnimation(
                    gapBetweenArcs: animatedLineStrokeWidth,
                    innerArcSpeedMultiplier: innerArcSpeedMultiplier,
                    outerArcSpeedMultiplier: outerArcSpeedMultiplier,
                    outerArcRadius: outerAnimatedArcRadius,
                    animatedLineStrokeWidth: animatedLineStrokeWidth,
                    accentColor: accentColor)
                : Icon(Icons.photo_camera, color: accentColor, size: iconSize),
          ),
          style: ElevatedButton.styleFrom(
            foregroundColor: accentColor,
            backgroundColor: primaryColor,
            shape:
                CircleBorder(side: BorderSide(color: primaryColor, width: 3)),
            padding: const EdgeInsets.all(0),
          ),
          onPressed: onPressed,
        ));
  }
}

class ShutterAnimation extends StatefulWidget {
  final double innerArcSpeedMultiplier;
  final double outerArcRadius;
  final double gapBetweenArcs;
  final double outerArcSpeedMultiplier;
  final double animatedLineStrokeWidth;
  final Icon manualSnapIcon;
  final Color accentColor;

  const ShutterAnimation(
      {Key? key,
      required this.innerArcSpeedMultiplier,
      required this.outerArcSpeedMultiplier,
      required this.animatedLineStrokeWidth,
      required this.outerArcRadius,
      required this.gapBetweenArcs,
      required this.accentColor,
      this.manualSnapIcon = const Icon(Icons.photo_camera)})
      : super(key: key);

  @override
  State<ShutterAnimation> createState() => _ShutterAnimationState();
}

class _ShutterAnimationState extends State<ShutterAnimation>
    with TickerProviderStateMixin {
  late final AnimationController _innerController;
  late final AnimationController _outerController;
  final ValueNotifier<double> _notifierInner = ValueNotifier(0);
  final ValueNotifier<double> _notifierOuter = ValueNotifier(0);

  @override
  void initState() {
    super.initState();
    Tween<double> _rotationTweenInner = Tween(begin: 2 * math.pi, end: 0);
    Tween<double> _rotationTweenOuter = Tween(begin: 0, end: 2 * math.pi);

    late final Animation<double> _animationOuter;
    late final Animation<double> _animationInner;

    _outerController = AnimationController(
      vsync: this,
      duration: Duration(
          milliseconds: (2000 * widget.outerArcSpeedMultiplier).toInt()),
    );
    _animationOuter = _rotationTweenOuter.animate(_outerController)
      ..addListener(() {
        _notifierOuter.value = _animationOuter.value.toDouble();
      })
      ..addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          _outerController.repeat();
        } else if (status == AnimationStatus.dismissed) {
          _outerController.forward();
        }
      });

    _innerController = AnimationController(
      vsync: this,
      duration: Duration(
          milliseconds: (3500 * widget.innerArcSpeedMultiplier).toInt()),
    );
    _animationInner = _rotationTweenInner.animate(_innerController)
      ..addListener(() {
        _notifierInner.value = _animationInner.value.toDouble();
      })
      ..addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          _innerController.repeat();
        } else if (status == AnimationStatus.dismissed) {
          _innerController.forward();
        }
      });

    _outerController.forward();
    _innerController.forward();
  }

  @override
  void dispose() {
    _innerController.dispose();
    _outerController.dispose();
    _notifierInner.dispose();
    _notifierOuter.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    var buttonSize = widget.outerArcRadius + widget.animatedLineStrokeWidth;
    return Center(
      child: Stack(
        children: [
          CustomPaint(
              painter: ArcPainter(
                  widget.outerArcRadius,
                  widget.animatedLineStrokeWidth,
                  270,
                  170,
                  widget.accentColor,
                  _notifierOuter),
              size: Size(buttonSize, buttonSize)),
          CustomPaint(
              painter: ArcPainter(
                  widget.outerArcRadius -
                      widget.animatedLineStrokeWidth -
                      widget.gapBetweenArcs,
                  widget.animatedLineStrokeWidth,
                  30,
                  150,
                  widget.accentColor,
                  _notifierInner),
              size: Size(buttonSize, buttonSize))
        ],
      ),
    );
  }
}

class ArcPainter extends CustomPainter {
  ValueNotifier<double> notifier;
  double radius;
  double strokeWidth;
  double arcStart;
  double arcEnd;
  Color color;
  var circlePaint = Paint();

  ArcPainter(this.radius, this.strokeWidth, this.arcStart, this.arcEnd,
      this.color, this.notifier)
      : super(repaint: notifier) {
    circlePaint
      ..color = color
      ..strokeWidth = strokeWidth
      ..style = PaintingStyle.stroke
      ..strokeCap = StrokeCap.round;
  }

  double degreeToRadian(double degree) {
    return degree * math.pi / 180;
  }

  double radianToDegree(double radian) {
    return radian * 180 / math.pi;
  }

  @override
  void paint(Canvas canvas, Size size) {
    Offset center = Offset(size.width / 2, size.height / 2);

    canvas.drawArc(
        Rect.fromCircle(
          center: center,
          radius: radius,
        ),
        degreeToRadian(arcStart) + notifier.value,
        degreeToRadian(arcEnd),
        false,
        circlePaint);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
