import 'dart:ui';

import 'package:json_annotation/json_annotation.dart';
import '../sdk_utils.dart';

part 'common_platform.g.dart';

/// Barcode document format
enum BarcodeDocumentFormat {
  /// American Association of Motor Vehicle Administrators barcode document */
  AAMVA,

  /// Boarding pass barcode document */
  BOARDING_PASS,

  /// German medical plan barcode document */
  DE_MEDICAL_PLAN,

  /// German medical certificate barcode document */
  MEDICAL_CERTIFICATE,

  /// ID Card barcode document */
  ID_CARD_PDF_417,

  /// SEPA barcode document */
  SEPA,

  /// Swiss QR barcode document */
  SWISS_QR,

  /// VCard barcode document */
  VCARD,

  /// GS1 barcode document */
  GS1,
}

/// Barcode document format
enum BarcodeFormat {
  /// Aztec barcode type */
  AZTEC,

  /// CODABAR barcode type */
  CODABAR,

  /// CODE_25 barcode type */
  CODE_25,

  /// CODE_39 barcode type */
  CODE_39,

  /// CODE_93 barcode type */
  CODE_93,

  /// CODE_128 barcode type */
  CODE_128,

  /// DATA_MATRIX barcode type */
  DATA_MATRIX,

  /// EAN_8 barcode type */
  EAN_8,

  /// EAN_13 barcode type */
  EAN_13,

  /// ITF barcode type */
  ITF,

  /// PDF_417 barcode type */
  PDF_417,

  /// QR_CODE barcode type */
  QR_CODE,

  /// MICRO_QR_CODE barcode type */
  MICRO_QR_CODE,

  /// RSS_14 barcode type */
  RSS_14,

  /// RSS_EXPANDED barcode type */
  RSS_EXPANDED,

  /// UPC_A barcode type */
  UPC_A,

  /// UPC_E barcode type */
  UPC_E,

  /// MSI Plessey barcode type */
  MSI_PLESSEY,

  /// IATA (2 of 5) barcode type */
  IATA_2_OF_5,

  /// INDUSTRIAL (2 of 5) barcode type */
  INDUSTRIAL_2_OF_5,

  /// USPS Intelligent Mail, a.k.a. USPS OneCode, USPS-STD-11 */
  USPS_INTELLIGENT_MAIL,

  /// Royal Mail Four-State Customer Code, a.k.a. RM4SCC, CBC, BPO 4 State Code */
  ROYAL_MAIL,

  /// Japan Post Four-State Barcode */
  JAPAN_POST,

  /// Royal TNT Post Four-State Barcode, a.k.a. KIX, Klant IndeX */
  ROYAL_TNT_POST,

  /// Australia Post Four-State Customer Code */
  AUSTRALIA_POST,

  /// GS1 DataBar Limited */
  DATABAR_LIMITED,
}

/// A filter for extended EAN and UPC barcodes.
enum BarcodesExtensionFilter {
  /// EAN and UPC codes are not filtered. Both are returned regardless if they have an extension or not. */
  NO_FILTER,

  /// Only EAN and UPC codes with extensions are returned. */
  ONLY_WITH_EXTENSIONS,

  /// Only EAN and UPC codes without extensions are returned. */
  ONLY_WITHOUT_EXTENSIONS,
}

/// Camera module to use
enum CameraModule {
  /// Front camera */
  FRONT,

  /// Back camera */
  BACK,

  /// The back camera with the widest available lens. iOS only. */
  BACK_WIDEST,
}

/// MSI plessey checksum algorithm
enum MSIPlesseyChecksumAlgorithm {
  /// Not use checksum */
  NONE,

  /// Mod10 checksum algorithm */
  MOD_10,

  /// Mod11IBM checksum algorithm */
  MOD_11_IBM,

  /// Mod11NCR checksum algorithm */
  MOD_11_NCR,

  /// Mod1010 checksum algorithm */
  MOD_1010,

  /// Mod1110IBM checksum algorithm */
  MOD_1110_IBM,

  /// Mod1110NCR checksum algorithm */
  MOD_1110_NCR,
}

/// Barcode scanner engine mode
enum EngineMode {
  /// Recommended barcode scanning mode. Used by default */
  NEXT_GEN,

  /// Legacy barcode scanning mode. */
  LEGACY,
}

/// Applied interface orientation
enum OrientationLockMode {
  /// Do not restrict interface orientation */
  NONE,

  /// Portrait screen orientations only */
  PORTRAIT,

  /// Landscape screen orientations only */
  LANDSCAPE,
}

/// Finder aspect ratio
@JsonSerializable(
    includeIfNull: false,
    createFactory: true,
    createToJson: true,
    explicitToJson: true)
class AspectRatio {
  /// The width component of the aspect ratio.
  double width;

  /// The height component of the aspect ratio.
  double height;

  AspectRatio({
    required this.width,
    required this.height,
  });

  Map<String, dynamic> toJson() => _$AspectRatioToJson(this);

  factory AspectRatio.fromJson(Map<String, dynamic> json) =>
      _$AspectRatioFromJson(json);
}

/// Standard size object
@JsonSerializable(
    includeIfNull: false,
    createFactory: true,
    createToJson: true,
    explicitToJson: true)
class Size {
  /// Width parameter
  double width;

  /// Height parameter
  double height;

  Size({
    required this.width,
    required this.height,
  });

  Map<String, dynamic> toJson() => _$SizeToJson(this);

  factory Size.fromJson(Map<String, dynamic> json) => _$SizeFromJson(json);
}

/// The expected density of QR codes in an image.
enum CodeDensity {
  /// Up to 6 QR codes per image. */
  LOW,

  /// Up to 24 QR codes per image. */
  HIGH,
}

/// Represents camera preview modes
enum CameraPreviewMode {
  /// In this mode camera preview frames will be downscaled to the layout view size - full preview frame content will be visible, but unused edges could be appeared in the preview layout. */
  FIT_IN,

  /// In this mode camera preview frames fill the layout view - the preview frames may contain additional content on the edges that was not visible in the preview layout. */
  FILL_IN,
}

/// Defines the possible barcode image sources.
enum BarcodeImageGenerationType {
  /// No barcode image generation. */
  NONE,

  /// Generating barcode image from the last video frame. Faster, but has lower resolution. */
  VIDEO_FRAME,

  /// Generating barcode image from separately captured frame. Higher resolution, but might have a slight delay. */
  CAPTURED_IMAGE,
}

/// Barcode scanner engine mode
enum BarcodeOverlayTextFormat {
  /// Show only barcode overlay frame. */
  NONE,

  /// Show barcode value with extention. */
  CODE,

  /// Show barcode value with barcode format. */
  CODE_AND_TYPE,
}

/// Barcode Selection Overlay configuration
@JsonSerializable(
    includeIfNull: false,
    createFactory: false,
    createToJson: true,
    explicitToJson: true)
class SelectionOverlayConfiguration {
  /// Whether the barcode selection overlay is enabled or not.
  bool? overlayEnabled;

  /// Whether the barcode is selected automatically when being detected or not.
  bool? automaticSelectionEnabled;

  /// Define the way of how to show barcode data with selection overlay.
  BarcodeOverlayTextFormat? textFormat;

  /// The color of the polygon in the selection overlay.
  @JsonKey(toJson: toColorString)
  Color? polygonColor;

  /// The color of the text in the selection overlay.
  @JsonKey(toJson: toColorString)
  Color? textColor;

  /// The color of the texts background in the selection overlay.
  @JsonKey(toJson: toColorString)
  Color? textContainerColor;

  /// The color of the polygon in the selection overlay, when highlighted.
  @JsonKey(toJson: toColorString)
  Color? highlightedPolygonColor;

  /// The color of the text in the selection overlay, when highlighted.
  @JsonKey(toJson: toColorString)
  Color? highlightedTextColor;

  /// The color of the texts background in the selection overlay, when highlighted.
  @JsonKey(toJson: toColorString)
  Color? highlightedTextContainerColor;

  SelectionOverlayConfiguration({
    this.overlayEnabled,
    this.automaticSelectionEnabled,
    this.textFormat,
    this.polygonColor,
    this.textColor,
    this.textContainerColor,
    this.highlightedPolygonColor,
    this.highlightedTextColor,
    this.highlightedTextContainerColor,
  });

  Map<String, dynamic> toJson() => _$SelectionOverlayConfigurationToJson(this);
}

/// Confirmation Dialog configuration
@JsonSerializable(
    includeIfNull: false,
    createFactory: false,
    createToJson: true,
    explicitToJson: true)
class ConfirmationDialogConfiguration {
  /// Defines, if the confirmation dialog should be displayed or not before returing the results to the delegate. Defaults to False.
  bool? resultWithConfirmationEnabled;

  /// The text format of the result dialog. Defaults to TYPE_AND_CODE.
  BarcodeDialogFormat? dialogTextFormat;

  /// The style of the confirmation dialog. iOS only.
  DialogStyle? confirmationDialogStyle;

  /// The title of the confirmation dialog confirm button.
  String? confirmButtonTitle;

  /// The style of the confirmation dialogs confirm button. iOS only.
  DialogButtonStyle? confirmationDialogConfirmButtonStyle;

  /// The title of the confirmation dialog retry button.
  String? retryButtonTitle;

  /// The style of the confirmation dialogs retry button. iOS only.
  DialogButtonStyle? confirmationDialogRetryButtonStyle;

  /// The title of the confirmation dialog.
  String? dialogTitle;

  /// The message text of the confirmation dialog.
  String? dialogMessage;

  /// The accent color of buttons on a confirmation dialog. Android only.
  @JsonKey(toJson: toColorString)
  Color? dialogButtonsAccentColor;

  /// Allows to set if the confirm button should be filled. Defaults to TRUE. Android only.
  bool? confirmButtonFilled;

  /// Allows to set a text color of the filled button. See `confirmationDialogConfirmButtonFilled`. Android only.
  @JsonKey(toJson: toColorString)
  Color? confirmButtonFilledTextColor;

  ConfirmationDialogConfiguration({
    this.resultWithConfirmationEnabled,
    this.dialogTextFormat,
    this.confirmationDialogStyle,
    this.confirmButtonTitle,
    this.confirmationDialogConfirmButtonStyle,
    this.retryButtonTitle,
    this.confirmationDialogRetryButtonStyle,
    this.dialogTitle,
    this.dialogMessage,
    this.dialogButtonsAccentColor,
    this.confirmButtonFilled,
    this.confirmButtonFilledTextColor,
  });

  Map<String, dynamic> toJson() =>
      _$ConfirmationDialogConfigurationToJson(this);
}

/// The font name and size. iOS only.
@JsonSerializable(
    includeIfNull: false,
    createFactory: false,
    createToJson: true,
    explicitToJson: true)
class Font {
  /// The font name. defaults to SYSTEM.
  String fontName;

  /// The font size. defaults 17.0 .
  double fontSize;

  Font({
    required this.fontName,
    required this.fontSize,
  });

  Map<String, dynamic> toJson() => _$FontToJson(this);
}

/// The blur effect style. iOS only.
enum BlurEffect {
  /// The area of the view is lighter than the underlying view. */
  EXTRA_LIGHT,

  /// The area of the view is the same approximate lightness of the underlying view. */
  LIGHT,

  /// The area of the view is darker than the underlying view. */
  DARK,

  /// A regular blur style that adapts to the user interface style. */
  REGULAR,

  /// A blur style for making content more prominent that adapts to the user interface style. */
  PROMINENT,

  /// An adaptable blur effect that creates the appearance of an ultra-thin material. iOS13+. */
  SYSTEM_ULTRA_THIN_MATERIAL,

  /// An adaptable blur effect that creates the appearance of a thin material. iOS13+. */
  SYSTEM_THIN_MATERIAL,

  /// An adaptable blur effect that create the appearance of a material with normal thickness. Defaults on iOS13 and more. iOS13+. */
  SYSTEM_MATERIAL,

  /// An adaptable blur effect that creates the appearance of a material that's thicker than normal. iOS13+. */
  SYSTEM_THICK_MATERIAL,

  /// An adaptable blur effect that creates the appearance of the system chrome. iOS13+. */
  SYSTEM_CHROME_MATERIAL,

  /// A blur effect that creates the appearance of an ultra-thin material and is always light. iOS13+. */
  SYSTEM_ULTRA_THIN_MATERIAL_LIGHT,

  /// A blur effect that creates the appearance of a thin material and is always light. iOS13+. */
  SYSTEM_THIN_MATERIAL_LIGHT,

  /// A blur effect that creates the appearance of a material with normal thickness and is always light. iOS13+. */
  SYSTEM_MATERIAL_LIGHT,

  /// A blur effect that creates the appearance of a material that’s thicker than normal and is always light. iOS13+. */
  SYSTEM_THICK_MATERIAL_LIGHT,

  /// A blur effect that creates the appearance of the system chrome and is always light. iOS13+ */
  SYSTEM_CHROME_MATERIAL_LIGHT,

  /// A blur effect that creates the appearance of an ultra-thin material and is always dark. iOS13+ */
  SYSTEM_ULTRA_THIN_MATERIAL_DARK,

  /// A blur effect that creates the appearance of a thin material and is always dark. iOS13+ */
  SYSTEM_THIN_MATERIAL_DARK,

  /// A blur effect that creates the appearance of a material with normal thickness and is always dark. iOS13+. */
  SYSTEM_MATERIAL_DARK,

  /// A blur effect that creates the appearance of a material that’s thicker than normal and is always dark. iOS13+. */
  SYSTEM_THICK_MATERIAL_DARK,

  /// A blur effect that creates the appearance of the system chrome and is always dark. iOS13+ */
  SYSTEM_CHROME_MATERIAL_DARK,
}

/// Defines a range for zooming
@JsonSerializable(
    includeIfNull: false,
    createFactory: false,
    createToJson: true,
    explicitToJson: true)
class ZoomRange {
  /// The minimum zoom scale. Defaults to 1.0.
  double? minZoom;

  /// The maximum zoom scale. Defaults to 12.0.
  double? maxZoom;

  ZoomRange({
    this.minZoom,
    this.maxZoom,
  });

  Map<String, dynamic> toJson() => _$ZoomRangeToJson(this);
}

/// Configuration for the dialog/alert style
@JsonSerializable(
    includeIfNull: false,
    createFactory: false,
    createToJson: true,
    explicitToJson: true)
class DialogStyle {
  /// The Color of the screen-covering backdrop view.
  @JsonKey(toJson: toColorString)
  Color screenBackgroundColor;

  /// The general background color of the actual dialog view.
  @JsonKey(toJson: toColorString)
  Color dialogBackgroundColor;

  /// The visual effect of the dialogs background.
  BlurEffect dialogBackgroundEffect;

  /// The corner radius of the dialog.
  double cornerRadius;

  /// The color of the dialogs title.
  @JsonKey(toJson: toColorString)
  Color titleColor;

  /// The font of the dialogs title
  Font titleFont;

  /// The color of the dialogs message.
  @JsonKey(toJson: toColorString)
  Color messageColor;

  /// The font of the dialogs message.
  Font messageFont;

  /// The color of the separators around the dialogs button area.
  @JsonKey(toJson: toColorString)
  Color separatorColor;

  /// The width of the separators in points.
  double separatorWidth;

  DialogStyle({
    required this.screenBackgroundColor,
    required this.dialogBackgroundColor,
    required this.dialogBackgroundEffect,
    required this.cornerRadius,
    required this.titleColor,
    required this.titleFont,
    required this.messageColor,
    required this.messageFont,
    required this.separatorColor,
    required this.separatorWidth,
  });

  Map<String, dynamic> toJson() => _$DialogStyleToJson(this);
}

/// Configuration for the dialogs/alerts OK button style.
@JsonSerializable(
    includeIfNull: false,
    createFactory: false,
    createToJson: true,
    explicitToJson: true)
class DialogButtonStyle {
  /// The font of the button title.
  Font? font;

  /// The color of the buttons title while not pressed.
  @JsonKey(toJson: toColorString)
  Color? textColor;

  /// The color of the buttons title while pressed.
  @JsonKey(toJson: toColorString)
  Color? highlightedTextColor;

  /// The background color of the button while not pressed.
  @JsonKey(toJson: toColorString)
  Color? backgroundColor;

  /// The background color of the button while pressed.
  @JsonKey(toJson: toColorString)
  Color? highlightedBackgroundColor;

  DialogButtonStyle({
    this.font,
    this.textColor,
    this.highlightedTextColor,
    this.backgroundColor,
    this.highlightedBackgroundColor,
  });

  Map<String, dynamic> toJson() => _$DialogButtonStyleToJson(this);
}

/// Mode for document detection.
enum DocumentDetectorMode {
  /// Is the recommended mode, based on Machine Learning. Requires iOS 11.2+. */
  ML_BASED,

  /// Legacy mode, based on classic Computer Vision algorithms. */
  EDGE_BASED,
}

/// The barcode text format dialog.
enum BarcodeDialogFormat {
  /// Show the barcode value only. */
  CODE,

  /// Show the barcode format with value. */
  TYPE_AND_CODE,
}

/// The document quality status that comes from DocumentQualityAnalyzer
enum DocumentQuality {
  /// Cant recognize any document. */
  NO_DOCUMENT,

  /// Very poor document quality. */
  VERY_POOR,

  /// Poor document quality. */
  POOR,

  /// Reasonable document quality. */
  REASONABLE,

  /// Good document quality. */
  GOOD,

  /// Excellent document quality. */
  EXCELLENT,
}

/// The image filter types.
enum ImageFilterType {
  /// Passthrough filter. Does not alter the image. */
  NONE,

  /// Optimizes colors, contrast and brightness. Usecase: photos. */
  COLOR,

  /// Standard grayscale filter. Creates a grayscaled 8-bit image and optimizes contrast and dynamic range. */
  GRAYSCALE,

  /// Standard binarization filter with contrast optimization. Creates a grayscaled 8-bit image with mostly black or white pixels. Usecase: Preparation for optical character recognition. */
  BINARIZED,

  /// Fixes white-balance and cleans up the background. Usecase: images of paper documents. */
  COLOR_DOCUMENT,

  /// A filter for binarizing an image. Creates an 8-bit image with pixel values set to eiter 0 or 255. Usecase: Preparation for optical character recognition. */
  PURE_BINARIZED,

  /// Cleans up the background and tries to preserve photos within the image. Usecase: magazine pages, flyers. */
  BACKGROUND_CLEAN,

  /// Black and white filter with background cleaning. Creates a grayscaled 8-bit image with mostly black or white pixels. Usecase: Textual documents or documents with black and white illustrations. */
  BLACK_AND_WHITE,

  /// A filter for black and white conversion using OTSU binarization. */
  OTSU_BINARIZATION,

  /// A filter for black and white conversion primary used for low-contrast documents. */
  DEEP_BINARIZATION,

  /// A filter that enhances edges in low-contrast documents. */
  EDGE_HIGHLIGHT,

  /// Binarization filter primary inteded to use on low-contrast documents with heavy shadows. */
  LOW_LIGHT_BINARIZATION,

  /// Binarization filter primary intended to use on low-contrast documents with heavy shadows. */
  LOW_LIGHT_BINARIZATION_2,

  /// Binarization filter for poor quality printed papers. */
  SENSITIVE_BINARIZATION,

  /// Standard grayscale filter. Creates a grayscaled 8-bit image. */
  PURE_GRAY,
}

/// The prioritization of still image quality and capturing speed. Has no effect on devices prior to iOS 13.0. iOS only.
enum CapturePhotoQualityPrioritization {
  /// Captures a still image at the highest possible speed. The quality of the image may be degraded. iOS only. */
  SPEED,

  /// Balances capturing speed and image quality equally. This is the default value. */
  BALANCED,

  /// Captures a still image with the best possible quality in terms of noise, frozen motion and detail in low light. The speed of the capturing might be significantly reduced. */
  QUALITY,
}

/// The page size of the pdf document.
enum PageSize {
  /// Letter page size. */
  LETTER,

  /// Legal page size. */
  LEGAL,

  /// A3 page size. */
  A3,

  /// A4 page size. */
  A4,

  /// A5 page size. */
  A5,

  /// B4 page size. */
  B4,

  /// B5 page size. */
  B5,

  /// Executive page size. */
  EXECUTIVE,

  /// US4x6 page size. */
  US4x6,

  /// US4x8 page size. */
  US4x8,

  /// US5x7 page size. */
  US5x7,

  /// COMM10 page size. */
  COMM10,

  /// Custom page size. From the Image. */
  CUSTOM,
}

/// The page orientation of the pdf document.
enum PageDirection {
  /// Portrait page orientation. */
  PORTRAIT,

  /// Landscape page orientation. */
  LANDSCAPE,

  /// Taken from the original image */
  AUTO,
}

/// The page orientation of the pdf document.
enum OcrEngine {
  /// Slow but powerful OCR engine. Supports non-Latin languages. */
  TESSERACT,

  /// Fast and accurate OCR engine. Supports only Latin languages. */
  SCANBOT_OCR,
}
